@@ -441,8 +441,8 @@ use L2hos;
 use vars qw(%prefs %cfg %newcfg);
 
 # This is the central place to modify the release name and date!!!
-my $RELEASE = '2020.2';
-my $VERSION = 'Released July 1, 2020';
+my $RELEASE = '2021.2';
+my $VERSION = 'Released July 1, 2021';
 
 # --------------------------------------------------------------------------
 # Open log
@@ -1444,15 +1444,15 @@ if(1) {
 }
 
 # --------------------------------------------------------------------------
-# PPMQUANT
+# PNMQUANT
 # --------------------------------------------------------------------------
 
-$newcfg{'PPMQUANT'} = '';
+$newcfg{'PNMQUANT'} = '';
 
 if(1) {
-  my $ppmquant = &find_prog(&get_name('PPMQUANT',1));
-  if($ppmquant) {
-    $newcfg{'PPMQUANT'} = $ppmquant;
+  my $pnmquant = &find_prog(&get_name('PNMQUANT',1));
+  if($pnmquant) {
+    $newcfg{'PNMQUANT'} = $pnmquant;
   }
 }
 
