@@ -1,38 +1,37 @@
 # -*- perl -*-
 #
-### File: html4.0.pl
-### Language definitions for HTML 4.0
+### File: html5.pl
+### Language definitions for HTML 5
 
 
-$DOCTYPE = "-//W3C//DTD HTML 4.0 Transitional";
-$DTD_URL = "http://www.w3.org/TR/PR-html40/loose.dtd";
-#$DOCTYPE = "-//W3C//DTD HTML 4.0";
-#$DTD_URL = " http://www.w3.org/DTD/HTML4-strict.dtd";
+$DOCTYPE = "HTML";
+$DTDcomment = "<!DOCTYPE HTML>\n";
 
 $USING_STYLES = 1;
-$math_class = ' ALIGN="CENTER" CLASS="mathdisplay"';
+$math_class = ' CLASS="displaymath"';
 $eqno_class = ' CLASS="eqno"';
 
 # Note that htmlx.x.pl prior modules are *NOT*already loaded.
 
-$declarations{'tiny'} = '<SMALL CLASS="TINY"></SMALL>';
-$declarations{'Tiny'} = '<SMALL CLASS="XTINY"></SMALL>';
-$declarations{'scriptsize'} = '<SMALL CLASS="SCRIPTSIZE"></SMALL>';
-$declarations{'small'} = '<SMALL CLASS="SMALL"></SMALL>';
-$declarations{'Small'} = '<SMALL CLASS="FOOTNOTESIZE"></SMALL>';
-$declarations{'SMALL'} = '<SMALL CLASS="SCRIPTSIZE"></SMALL>';
-$declarations{'footnotesize'} = '<SMALL CLASS="FOOTNOTESIZE"></SMALL>';
-$declarations{'large'} = '<BIG CLASS="LARGE"></BIG>';
-$declarations{'Large'} = '<BIG CLASS="XLARGE"></BIG>';
-$declarations{'LARGE'} = '<BIG CLASS="XXLARGE"></BIG>';
-$declarations{'huge'} = '<BIG CLASS="HUGE"></BIG>';
-$declarations{'Huge'} = '<BIG CLASS="XHUGE"></BIG>';
+$declarations{'tiny'} = '<SPAN CLASS="TINY"></SPAN>';
+$declarations{'Tiny'} = '<SPAN CLASS="XTINY"></SPAN>';
+$declarations{'scriptsize'} = '<SPAN CLASS="SCRIPTSIZE"></SPAN>';
+$declarations{'small'} = '<SPAN CLASS="SMALL"></SPAN>';
+$declarations{'Small'} = '<SPAN CLASS="FOOTNOTESIZE"></SPAN>';
+$declarations{'SMALL'} = '<SPAN CLASS="SCRIPTSIZE"></SPAN>';
+$declarations{'footnotesize'} = '<SPAN CLASS="FOOTNOTESIZE"></SPAN>';
+$declarations{'large'} = '<SPAN CLASS="LARGE"></SPAN>';
+$declarations{'Large'} = '<SPAN CLASS="XLARGE"></SPAN>';
+$declarations{'LARGE'} = '<SPAN CLASS="XXLARGE"></SPAN>';
+$declarations{'huge'} = '<SPAN CLASS="HUGE"></SPAN>';
+$declarations{'Huge'} = '<SPAN CLASS="XHUGE"></SPAN>';
 
 $declarations{'sl'} = '<I CLASS="slanted"></I>';
 $declarations{'slshape'} = '<I CLASS="slanted"></I>';
-$declarations{'sf'} = '<I CLASS="sans"></I>';
-$declarations{'sffamily'} = '<I CLASS="sans"></I>';
-$declarations{'scshape'} = '<I CLASS="smcaps"></I>';
+$declarations{'sf'} = '<SPAN style="font-family:sans-serif"></SPAN>';
+$declarations{'sffamily'} = '<SPAN style="font-family:sans-serif"></SPAN>';
+$declarations{'scshape'} = '<SPAN style="font-variant:small-caps"></SPAN>';
+$declarations{'tt'} = '<SPAN style="font-family:monospace"></SPAN>';
 
 ### Allow for alignment to work
 
@@ -43,16 +42,6 @@ sub do_env_flushleft { &declared_env('flushleft', @_) }
 sub do_env_raggedright { &declared_env('flushleft', @_) }
 sub do_env_raggedleft { &declared_env('flushright', @_) }
 
-sub do_cmd_centering { 
-    # MRO *opentags has no effect
-    local($_)= @_; &declared_env('center', $_) }
-sub do_cmd_raggedright {
-    # MRO *opentags has no effect
-    local($_) = @_; &declared_env('flushleft', $_) }
-sub do_cmd_raggedleftt {
-    # MRO *opentags has no effect
-    local($_) = @_; &declared_env('flushright', $_) }
-
 sub do_cmd_underline {
     &styled_text_chunk('U','','text','decoration','underline','',@_)}
 
@@ -79,7 +68,7 @@ $percent_type = "\^\\d+(%|\\%|$percent_mark)?\$";
 $coord_type = "\^\\s*(\$|\\d+(\\s*,\\s*\\d+)+\\s*\$)";  # comma-separated list of numbers
 $pixel_type = "\^\\d+\$";
 $length_type = "\^\\d+\%?\$";
-$name_type = "\^[A-Za-z]([A-Za-z\\d\\-\\.]*\$";
+$name_type = "\^[A-Za-z][A-Za-z\\d\\-\\.]*\$";
 $string_type = $URL_type = $script_type = $ISOdate_type = $Internet_type =
 	$CDATA_type = "\^.*\$";
 $id_list_type = "\^\\s*\$name_type(\\s+$name_type)*\\s*\$";
@@ -655,7 +644,7 @@ $wrap_parbox_rx = "(\\\\begin$O\\d+${C}tex2html_deferred$O\\d+$C)?"
 
 sub translate_colspec {
     local($colspec,$celltag) = @_;
-    local($cellopen) = "<$celltag ALIGN";
+    local($cellopen) = "<$celltag CLASS";
     local($cellclose) = "</$celltag>\n";
     local($len,$pts,@colspec,$char,$cols,$repeat,$celldata,$at_text,$after_text);
     local($frames, $rules, $prefix,$border)=('','','','');
@@ -702,7 +691,6 @@ sub translate_colspec {
 	} elsif ( $char =~ /^(p|m|b|t)$/ ) {
 	    local($valign);
 	    $valign = (($char eq "b")? "BOTTOM" : (($char eq "m")? "MIDDLE" : "TOP"));
-	    $valign = " VALIGN=\"${valign}\"";
 	    $colspec =~ s/^\s*\{([^\}]*)\}\s*/$celldata=$1;''/e;
 
 	    local($pxs,$len) = &convert_length($celldata) if ($celldata);
@@ -713,7 +701,7 @@ sub translate_colspec {
 	    }
 	    if ($at_text) { $at_text = $art_br.$at_text; $after_text = $art_br; }
 	    push(@colspec
-		,"$cellopen=\"LEFT\"${valign}$celldata>$at_text$content_mark$after_text$cellclose");
+		,"$cellopen=\"LEFT ${valign}\" $celldata>$at_text$content_mark$after_text$cellclose");
 	    $at_text = $after_text = '';
 	    $cols++;
 
@@ -929,6 +917,7 @@ sub process_tabular {
     s/\\\\\s*\[([^]]+)\]/\\\\/g;  # TKM - get rid of [N.n pc] on end of rows...
     s/\\newline\s*\[([^]]+)\]/\\newline/g;
     s/\n\s*\n/\n/g;	# Remove empty lines (otherwise will have paragraphs!)
+    s/\\\\\s*\n\s*$par_rx\s*\n/\\\\\n/g; # don't start next row with a \par
     local($i,@colspec,$char,$cols,$cell,$htmlcolspec,$frames,$rules);
     local(@rows,@cols,$border,$frame);
     local($colspan,$cellcount);
@@ -1004,7 +993,7 @@ sub process_tabular {
     $rules  .= "r" if ( s/\\[hv]line//g );
     #RRM:  retain any \cline to gobble its argument
 
-    if ( $frames || $rules ) { $border = " BORDER=\"1\""; };
+    if ( $frames || $rules ) { $border = " BORDER"; };
 
     # convert \\s inside \parbox commands to \newline s;
     # catch nestings
@@ -1016,7 +1005,7 @@ sub process_tabular {
     if ($tab_width) {
 	if ($tab_width =~ /(\%|$percent_mark)/) { $tab_width = "\"$tab_width\"" }
 	elsif ($tab_width <= 1) {$tab_width = "\"".(100 x $tab_width)."\"" }
-	$tab_width = " WIDTH=$tab_width";
+	$tab_width = " WIDTH:$tab_width;";
     } else { $tab_width = '' }
 
 #    if ($TABLE_CAPTIONS) { # captions for super-tabular environment
@@ -1030,19 +1019,15 @@ sub process_tabular {
 
     if ($table_captions) {
 	$return = join('', (($cap_anchors)? "$cap_anchors\n" : '')
-		, "<TABLE$env_id$lang CELLPADDING=3"
-		, $border
-		, ($halign ? " ALIGN=\"$halign\"" : '')
-		, $tab_width, ">");
+		, "<TABLE$env_id$lang class=\"PAD $border\" style=\""
+		, $tab_width, "\">");
 	$return .= $table_captions;
 #	$return .= "\n<CAPTION$lang>$captions</CAPTION>";
 #	$captions = '';
 #	$cap_anchors = '';
     } else { 
-	$return = join('', "<TABLE$env_id$lang CELLPADDING=3"
-		, $border
-		, ($halign ? " ALIGN=\"$halign\"" : '')
-		, $tab_width, ">");
+	$return = join('', "<TABLE$env_id$lang class=\"PAD $border\" style=\""
+		, $tab_width, "\">");
     }
     $env_id = '';
 
@@ -1089,7 +1074,10 @@ sub process_tabular {
  
 	for ( $i = 0; $i <= $#colspec; $i++ ) {
 	    # skip this cell if it is covered by a \multirow
-	    next if ($has_multirow && @row_spec[$i] > 0);
+	    if ($has_multirow && @row_spec[$i] > 0) {
+	        shift(@cols);
+	        next;
+	    }
 
 	    $colspec = $colspec[$i];
 	    if (!($colspec =~ $content_mark)) {
@@ -1355,9 +1343,10 @@ sub do_env_tex2html_wrap_inline {
     } else { $_ }
 }
 
-# Allocate a fixed width for the equation-numbers:
-#$seqno = "\n<TD WIDTH=1 ALIGN=\"CENTER\">\n";
 
+
+# produces a <DIV class="equation">
+# eq no is a <SPAN class="FLOATRIGHT">
 sub do_env_equation {
     local($_) = @_;
     local($math_mode, $failed, $labels, $comment) = ("equation",'','');
@@ -1372,8 +1361,8 @@ sub do_env_equation {
     local($math_start,$math_end)= ($sbig,$ebig);
 
     local($eqno);
-    local($seqno) = join('',"\n<TD$eqno_class WIDTH=10 ALIGN=\""
-                         , (($EQN_TAGS =~ /L/)? 'LEFT': 'RIGHT')
+    local($seqno) = join('',"<SPAN class=\""
+                         , (($EQN_TAGS =~ /L/)? 'FLOATLEFT': 'FLOATRIGHT')
 		         , "\">\n");
     do { # get the equation number
 	$global{'eqn_number'}++;
@@ -1396,53 +1385,40 @@ sub do_env_equation {
 	$eqno = join('',$EQNO_START, $eqno, $EQNO_END)
     } else { $eqno = '&nbsp;' } # spacer, when no numbering
 
-    # include the equation-number, using a <TABLE>
-    local($halign) = " ALIGN=\"CENTER\"" unless $FLUSH_EQN;
+    # include the equation-number, using a <DIV class="equation">
+    local($halign) = " &nbsp; " unless $FLUSH_EQN;
     local($lang) = &get_current_language();
     if ($EQN_TAGS =~ /L/) {
 	# equation number on left
 	($math_start,$math_end) =
-	    ("\n<TABLE$env_id$lang WIDTH=\"100%\" ALIGN=\"CENTER\""
-		. (($border)? " BORDER=\"$border\"" : '')
-		. (($attribs)? " $attribs" : '')
-		. ">\n<TR VALIGN=\"MIDDLE\">". $seqno . $eqno
-		. "</TD>\n<TD$halign NOWRAP>$sbig"
-	    , "$ebig</TD>\n</TR></TABLE>");
+	    ($seqno . $eqno . "</SPAN>"
+		. "$sbig $halign"
+	    , "$ebig\n");
 	$border = $attribs = $env_id = '';
     } else {
 	# equation number on right
 	($math_start,$math_end) =
-	    ("\n<TABLE$env_id$lang WIDTH=\"100%\" ALIGN=\"CENTER\""
-		. (($border)? " BORDER=\"$border\"" : '')
-		. (($attribs)? " $attribs" : '')
-		. ">\n<TR VALIGN=\"MIDDLE\"><TD></TD>"
-		. "<TD$halign NOWRAP>$sbig"
-	    , $ebig .'</TD>'. $seqno . $eqno ."</TD></TR>\n</TABLE>");
+	    ("$halign $sbig"
+	    , $ebig .' '. $seqno . $eqno ."</SPAN>\n");
 	$border = $attribs = $env_id = '';
     }
 
     ($labels, $comment, $_) = &process_math_env($math_mode,$_);
     if ($failed) {
-	$_ = join ('', $comment, $labels, $math_start
-	    , &process_undefined_environment('displaymath', $id, $saved)
-	    , $math_end );
+	$_ = join ('', $comment, $math_start
+		   , &process_math_in_latex("indisplay", '', '', $saved)
+		   , $math_end );
     } else {
 	s/^[ \t]*\n?/\n/; s/\n?[ \t]*$/\n/;
 	$_ = join('', $comment, $labels, $math_start, $_, $math_end );
     }
     if ($border||($attribs)) { 
-	join('',"<BR>\n<DIV$math_class>\n"
+	join('',"\n<DIV$math_class>\n"
 	    , &make_table( $border, $attribs, '', '', '', $_ )
-	    , "\n<BR CLEAR=\"ALL\">");
-    } elsif ($failed) {
-	$falign = (($EQN_TAGS =~ /L/)? 'LEFT' : 'RIGHT') if ($eqno);
-	local($fclass) = $math_class;
-	$fclass =~ s/(ALIGN=\")[^"]*/$1$falign/;
-	join('',"<BR>\n<DIV$fclass>\n"
-	    , $_ , "\n<BR CLEAR=\"ALL\"></DIV><P></P>")
+	    , "\n");
     } else { 
-	join('', "<BR><P></P>\n<DIV$math_class>\n"
-	    , $_ ."\n</DIV><BR CLEAR=\"ALL\"><P></P>");
+	join('', "\n<DIV$math_class>\n"
+	    , $_ ."</DIV>");
     }
 }
 
@@ -1460,7 +1436,7 @@ sub do_env_displaymath {
 	if (($DISP_SCALE_FACTOR)&&($DISP_SCALE_FACTOR >= 1.2 ));
     ($labels, $comment, $_) = &process_math_env($math_mode,$_);
     if ($failed) {
-	$_ = &process_undefined_environment("displaymath", $id, $saved);
+	$_ = &process_math_in_latex("indisplay",'','',$saved);
 	s/^[ \t]*\n?/\n/; s/\n?[ \t]*$/\n/; 
 	$_ = (($comment.$labels)? "$comment$labels\n":''). $_;
     } else {
@@ -1468,12 +1444,12 @@ sub do_env_displaymath {
 	$_ = (($comment.$labels)? "$comment$labels\n":'').$sbig.$_.$ebig;
     }
     if ($border||($attribs)) {
-	join('',"<BR>\n<DIV$math_class>\n"
+	join('',"<DIV$math_class>\n"
             , &make_table( $border, $attribs, '', '', '', $_ )
-	    , "</DIV>\n<BR CLEAR=\"ALL\">");
+	    , "</DIV>\n");
     } else { 
-        join('',"<BR><P></P>\n<DIV$math_class>",$_
-            ,"</DIV><BR CLEAR=\"ALL\">\n<P></P>");
+        join('',"<DIV$math_class>",$_
+            ,"</DIV>\n");
     }
 }
 
@@ -1500,23 +1476,23 @@ sub do_env_eqnarray {
 #	||((/$htmlimage_rx|$htmlimage_pr_rx/)&&($& =~/thumb/))) {
 	# image of whole environment, no equation-numbers
 	$failed = 1;
-	$falign = (($EQN_TAGS =~ /L/)? 'LEFT' : 'RIGHT')
-	    unless $no_eqn_numbers;
+	local($fclass) = " class=" . (($EQN_TAGS =~ /L/)? 'FLOATLEFT': 'FLOATRIGHT');
+	if ($no_eqn_numbers) {
+	    $fclass = " class=CENTER";
+	}
 	$_ = join ('', $comment
 	    , &process_undefined_environment(
 		"eqnarray".(($no_eqn_numbers) ? "star" : '')
 		, $id, $saved));
-	local($fclass) = $math_class;
-	$fclass =~ s/(ALIGN=\")[^"]*/$1$falign/;
-	$_ = join('',"<P></P><DIV$fclass>"
-	    , $_, "</DIV><BR CLEAR=\"ALL\"><P></P>\n");
+	$_ = join('',"<DIV$fclass>"
+	    , $_, "</DIV>\n");
     } else {
 	$failed = 0;
 	s/$htmlimage_rx/$doimage = $&;''/eo ; # force an image
 	s/$htmlimage_pr_rx/$doimage .= $&;''/eo ; # force an image
 	local($sarray, $srow, $slcell, $elcell, $srcell, $ercell, $erow, $earray);
 	($sarray, $elcell, $srcell, $erow, $earray, $sempty) = ( 
-	    "\n<TABLE$env_id$lang CELLPADDING=\"0\" ALIGN=\"CENTER\""
+	    "\n<TABLE$env_id$lang CLASS=\"equation\""
 	    , "</TD>\n<TD ALIGN=\"CENTER\" NOWRAP>"
 	    , "</TD>\n<TD ALIGN=\"LEFT\" NOWRAP>"
 	    , "</TD></TR>", "\n</TABLE>", "</TD>\n<TD>" );
@@ -1543,7 +1519,7 @@ sub do_env_eqnarray {
 	$#rows-- if ( $rows[$#rows] =~ /^\s*$/ );
 	$return = join(''
 	    , (($border||($attribs))? '': "<BR>")
-	    , (($doimage)? '' : "\n<DIV ALIGN=\"CENTER\">")
+	    , (($doimage)? '' : "\n<DIV$math_class>")
 	    , (($labels)? $labels : "\n") , $comment, $sarray);
 	foreach (@rows) { # displaymath
 	    $eqno = '';
@@ -1666,9 +1642,9 @@ sub do_env_eqnarray {
     if ($border||($attribs)) { 
 	join('' #,"<BR>\n<DIV$math_class>"
 	    , &make_table( $border, $attribs, '', '', '', $_ )
-	    , "\n</DIV><P></P><BR CLEAR=\"ALL\">");
+	    , "\n</DIV>");
     } else {
-	join('', $_ ,"<BR CLEAR=\"ALL\"><P></P>");
+	join('', $_ ,"");
     }
 }
 
@@ -1710,18 +1686,14 @@ sub do_env_eqnarraystar {
     if ($border||($attribs)) { 
 	join('' #,"<BR>\n<DIV$math_class>"
 	    , &make_table( $border, $attribs, '', '', '', $_ )
-	    , "\n</DIV><P></P><BR CLEAR=\"ALL\">");
+	    , "\n</DIV>");
     } elsif ($failed) {
 	$_ =~ s/^[ \t]*\n?/\n/; 
 	$_ =~ s/\n?[ \t]*/\n/;
-	join('', "<BR><P></P>\n<DIV$math_class>"
-	    , $_ ,"\n</DIV><P></P><BR CLEAR=\"ALL\">");
-    } elsif ($_ =~ s!(</TABLE></DIV>)\s*(<BR[^>]*><P></P>)?\s*$!$1!si) {
-	join('', $_ ,"<BR>\n");
-    } elsif ($_ =~ m!<P></P>\s*$!si) { # below-display space present 
-	$_
+	join('', "<DIV>"	# padding included in image
+	    , $_ ,"\n</DIV>");	# so don't need $math_class
     } else {
-	join('', $_ ,"<BR CLEAR=\"ALL\"><P></P>");
+	$_;
     }
 }
 
